<?php
/**
 * Created by JetBrains PhpStorm.
 * User: admin
 * Date: 8/13/17
 * Time: 8:57 PM
 * To change this template use File | Settings | File Templates.
 */
class DSCF_DSF_ClientDashboardAssistant{

    const SETTINGS_PLUGIN_METADATA = "wpdevhub-plugin-metadata";

    /*
     * Checks whether or not the Client Dashboard is Installed
     */
    public static function isDimbalClientDashboardInstalled(){
        return is_plugin_active("wpdevhub-wcd/index.php");
    }

    /*
     * Filter hook to "plugin_row_meta" to add a Promo for the DCD in the Updater list of links
     */
    public static function registerUpdaterFilters(){

        if(current_user_can('update_plugins')){

            // Add any links as appropriate into the Plugin Meta Row table
            add_filter('plugin_row_meta', array('DSCF_DSF_ClientDashboardAssistant', 'callbackInjectPluginMetaRowLinks'), 10, 2);

            // Modify any pending Updates that need their update prompts adjusted
            add_action( 'site_transient_update_plugins', array( 'DSCF_DSF_ClientDashboardAssistant', 'callbackInjectUpdateTagline' ), 15 ); //make sure it runs after WP's

        }

    }

    /*
     * Callback function from the "plugin_row_meta" filter to add a promo for the DCD
     */
    public static function callbackInjectPluginMetaRowLinks($links, $pluginFile) {

        if($pluginFile == DSCF_DSF_Utilities::getBasePluginFile()){

            // Validate the user can in fact modify plugins
            if ( current_user_can('update_plugins') ) {

                $wcdInstalled = DSCF_DSF_ClientDashboardAssistant::isDimbalClientDashboardInstalled();

                if(empty($wcdInstalled)){
                    // DCD is not installed
                    $links[] = '<a href="http://www.wpdevhub.com" target="_blank">Install Client Dashboard</a>';
                }else{
                    // DCD is installed - Validate subscription

                    // Validate the Users Subscription Settings
                    $isThisPro = DSCF_DSF_Utilities::isPluginPro();
                    $validSubscription = self::getSubscriptionValidation();
                    if(empty($validSubscription)){
                        // Invalid Subscription

                        if(empty($isThisPro)){
                            // Free Plugin - Nothing to do as WP Update path should be provided - but provide a Subscribe soft prompt
                            $links[]='<a href="http://www.wpdevhub.com" target="_blank">Subscribe to Pro Version</a>';
                        }else{
                            // Pro Plugin - Need to remove Links and Replace with Downgrade or Subscribe Options
                            //$links[]='<a href="http://www.wpdevhub.com" target="_blank">Subscribe to Pro Updates</a>';
                            $links[]='<a href="http://www.wpdevhub.com" target="_blank">Downgrade to Free Version</a>';
                        }

                    }else{
                        // Valid Subscription

                        if(empty($isThisPro)){
                            // Free Plugin - Show an Upgrade Prompt as Well
                            $links[]='<a href="http://www.wpdevhub.com" target="_blank">Upgrade to Pro Version</a>';

                        }else{
                            // Pro Plugin - Nothing left to do as subscription is valid and upgrade link already set
                        }
                    }

                }

            }else{
                // User cannot modify plugins
            }
        }else{
            // Plugin file data not match
        }

        return $links;
    }

    /*
    * Checks if any updates are present - if so validates them against settings and changes the links as needed
    */
    public static function callbackInjectUpdateTagline($updates){

        //DSCF_DSF_Utilities::logMessage("Updates List: ".print_r($updates, true));

        if(isset($updates->response)){
            foreach($updates->response as $key=>$update){
                if($key === DSCF_DSF_Utilities::getBasePluginFile()){
                    // There is an Update for this Plugin.  Fire off the Adjustment Filter

                    remove_all_actions( 'after_plugin_row_' . $key );
                    add_action('after_plugin_row_' . $key, array( 'DSCF_DSF_ClientDashboardAssistant', 'callbackInjectPluginUpdateRow'), 9, 2 );

                }
            }
        }

        return $updates;
    }

    public static function callbackInjectPluginUpdateRow( $file, $plugin_data ) {

        // Validate the user can update plugins
        if ( !current_user_can('update_plugins') ) {
            return;
        }

        $version = DSCF_DSF_Utilities::getFromArray('new_version', $plugin_data, '####');

        $plugins_allowedtags = array('a' => array('href' => array(),'title' => array()),'abbr' => array('title' => array()),'acronym' => array('title' => array()),'code' => array(),'em' => array(),'strong' => array());
        $plugin_name = wp_kses( $plugin_data['Name'], $plugins_allowedtags );

        $info_url = "https://www.wpdevhub.com/info_url";

        $action_url = DSCF_DSF_Utilities::getFromArray('package', $plugin_data, '####');
        $message = "update now";

        // Validate the Users Subscription Settings
        $wcdInstalled = DSCF_DSF_ClientDashboardAssistant::isDimbalClientDashboardInstalled();
        $isThisPro = DSCF_DSF_Utilities::isPluginPro();
        $validSubscription = self::getSubscriptionValidation();

        if(empty($wcdInstalled)){
            // No DCD Installed - no reason to check subscription

            if(empty($isThisPro)){
                // Free Plugin - Display the standard style upgrade messaging
            }else{
                // Pro Plugin - Display the prompt to install the client dashboard
                $action_url = "https://www.wpdevhub.com/install_client_dashboard";
                $message = "Install Dimbal Client Dashboard";
            }

        }else{
            // DCD Installed

            if(empty($validSubscription)){
                // No valid subscription

                if(empty($isThisPro)){
                    // Free Plugin, no subscription.  Display standard upgrade message
                }else{
                    // Pro Plugin, no subscription.  Revise the upgrade message
                    $action_url = "https://www.wpdevhub.com/activate subscription";
                    $message = "Activate Dimbal Subscription";
                }

            }else{
                // Valid subscription

                if(empty($isThisPro)){
                    // Free Plugin, valid subscription.  Display standard upgrade message
                }else{
                    // Pro plugin, Valid subscription.  Display standard upgrade message
                }
            }
        }

        echo '<tr class="plugin-update-tr"><td colspan="3" class="plugin-update colspanchange"><div class="update-message notice inline notice-warning notice-alt"><p>';
        printf( 'There is a new version of %1$s available on Dimbal Software. <a href="%2$s" class="thickbox" title="%3$s">View version %4$s details</a> or <a href="%5$s">%6$s</a> to update.', $plugin_name, esc_url($info_url), esc_attr($plugin_name), $version, esc_url($action_url), $message );
        echo '</p></div></td></tr>';

    }


    /*
     * Doesn't actually validate against the server - that happens during updates
     * Really just a shortcut function to get saved data
     * Meaning - a malicious user can try to override this function but will ultimately cause an error when updates are attempted.
     */
    public static function getSubscriptionValidation(){
        if(class_exists('DSCF_DSF_ClientDashboard_Main')){
            return DSCF_DSF_ClientDashboard_Main::isSubscriptionActive();
        }
        return false;
    }

    /**************************** PLUGIN METADATA *************************/

    /*
     * Make sure that the meta data array elements are formatted properly
     */
    public static function formatPluginMetaData($metadata){
        if(!is_array($metadata)){
            $metadata=array();
        }
        $defaults = array(
            'wpdevhub-key'=>'',           // Ex: DCD
            'installed-version'=>'',    // Version installed
            'new-version'=>'',          // Version of the update available if applicable
            'level'=>''                 // Free or Pro
        );
        $metadata = array_merge($defaults, $metadata);
        return $metadata;
    }

    /*
     * Adds a single key value pair into a plugins metadata array
     */
    public static function addPluginMetaDataValue($plugin, $key, $value){
        $metadata = self::getPluginMetaData($plugin);
        $metadata[$key]=$value;
        self::savePluginMetaData($plugin, $metadata);
    }

    /*
     * Removes a single key value pair from a plugins metadata array
     */
    public static function removePluginMetaDataValue($plugin, $key){
        $metadata = self::getPluginMetaData($plugin);
        if(array_key_exists($key, $metadata)){
            unset($metadata[$key]);
        }
        self::savePluginMetaData($plugin, $metadata);
    }

    /*
     * Retrieves the metadata array for a single given plugin
     */
    public static function getPluginMetaData($plugin){
        $list = self::getPluginMetaDataList();

        $metadata = array();
        if(array_key_exists($plugin, $list)){
            $metadata = $list[$plugin];
        }

        return self::formatPluginMetaData($metadata);
    }

    /*
     * Saves the metadata array for a given plugin
     */
    public static function savePluginMetaData($plugin, $metadata){
        $list = self::getPluginMetaDataList();

        $metadata = self::formatPluginMetaData($metadata);
        $list[$plugin]=$metadata;

        self::savePluginMetaDataList($list);

    }

    /*
     * Removes an entire metatdata array for a given plugin
     */
    public static function removePluginMetaData($plugin){
        $list = self::getPluginMetaDataList();

        if(array_key_exists($plugin, $list)){
            unset($list[$plugin]);
        }

        self::savePluginMetaDataList($list);
    }

    public static function getPluginMetaDataList(){
        $list = get_option(self::SETTINGS_PLUGIN_METADATA, array());
        if(!is_array($list)){
            $list=array();
        }
        return $list;
    }

    public static function savePluginMetaDataList($list){
        // Note -- cannot use the Dimbal Settings framework here as we want a common keyname and not plugin specific
        update_option(self::SETTINGS_PLUGIN_METADATA, $list, false);
    }

    /**************************** END PLUGIN METADATA *************************/

}
