<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/12/15
 * Time: 11:23 PM
 *
 * This file controls the Widget Options for the Dimbal Software Poll Widgets
 *
 */
class DSCF_DSF_SupportForums_AddThreadFormWidget extends WP_Widget {

    public static $displayOptions = array(
        1=>'button_text'
    );

    public function __construct() {
        // widget actual processes
        parent::__construct(
            'DSCF_DSF_SupportForums_AddThreadFormWidget', // Base ID
            'Support Forums Add Thread Form', // Name
            array( 'description' => __( 'Add a form to allow users to start a new forum thread.', 'text_domain' ), ) // Args
        );
    }

    public function form( $instance ) {
        // outputs the options form on admin
        $title = __( 'Add Support Forum Thread' );
        $buttonText = __( 'Add New Thread' );
        if ( $instance ) {
            $title = esc_attr( $instance['title'] );
            $buttonText = esc_attr( $instance['button_text'] );
        }

        ?>
        <p>
        <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label>
        <input class="widefat" id="<?=($this->get_field_id( 'title' ))?>" name="<?=($this->get_field_name( 'title' ))?>" type="text" value="<?=($title)?>" />

        <label for="<?php echo $this->get_field_id( 'button_text' ); ?>"><?php _e( 'Button Text:' ); ?></label>
        <input class="widefat" id="<?=($this->get_field_id( 'button_text' ))?>" name="<?=($this->get_field_name( 'button_text' ))?>" type="text" value="<?=($buttonText)?>" />
        </p>
    <?php
    }

    public function update( $new_instance, $old_instance ) {
        // processes widget options to be saved
        $title = (array_key_exists('title',$new_instance))?$new_instance['title']:"";
        $instance['title'] = strip_tags($title);
        $title = (array_key_exists('button_text',$new_instance))?$new_instance['button_text']:"Add New Thread";
        $instance['button_text'] = strip_tags($title);
        return $instance;
    }

    public function widget( $args, $instance ) {
        // outputs the content of the widget

        // Make sure the framework is enabled (this will prevent the widget from display entirely)
        if(!DSCF_DSF_Utilities::isPluginEnabled()){
            return;
        }


        $threadValidation = true;
        //$threadValidation = DSCF_DSF_SupportForums_Main::checkThreadValidation();
        if(!$threadValidation){
            // Failed thread validation - do not show the form
            return;
        }

        $userObject = wp_get_current_user();
        if(empty($userObject) || empty($userObject->ID)){
            // Not a valid user - do not show the form
            return;
        }

        if(!DSCF_DSF_SupportForums_Main::canUserAddContent($userObject->ID)){
            // User posting too fast - do not show the form
            return;
        }

        ////////  At this point we are ok to start showing the widget form

        // Before the widget
        echo $args['before_widget'];

        // Title information
        if ( ! empty( $instance['title'] ) ) {
            echo $args['before_title'];
            echo esc_html( $instance['title'] );
            echo $args['after_title'];
        }

        ?>
        <form id="new_post" name="new_post" method="post" action="<?=(DSCF_DSF_SupportForums_Main::getAddThreadHandlerUrl())?>">

            <p><label for="title">Title:</label><br />
            <input type="text" name="title" id="title" value="" placeholder="Thread Title" /></p>

            <p><label for="Category">Category:</label><br />
            <?=(wp_dropdown_categories( "show_option_none=-------&tab_index=4&echo=0&taxonomy=DSCF_DSF_category" ))?></p>


            <p><label for="content">Content:</label><br />
            <textarea name="content" id="content"></textarea></p>

            <p><label for="tags">Tags:</label>
            <input type="text" value="" tabindex="5" size="16" name="tags" id="tags" /></p>

            <p align="right"><input type="submit" value="Publish Thread" tabindex="6" id="submit" name="submit" /></p>

            <input type="hidden" name="action" value="new_post" />
            <?=(wp_nonce_field( 'new-post' ))?>
        </form>

        <?php

        // After the widget
        echo $args['after_widget'];

    }
}
