<?php
/**
 * Created by JetBrains PhpStorm.
 * User: admin
 * Date: 3/3/17
 * Time: 4:56 PM
 * To change this template use File | Settings | File Templates.
 */
class DSCF_DSF_SupportForums_Thread extends DSCF_DSF_StandardCustomPostType {

    const KEYNAME = "dsf_thread";
    const KEYNAME_CATEGORY = "dsf_thread_category";
    const KEYNAME_TAG = "dsf_thread_tag";

    public function __construct(){

        parent::__construct();

    }

    public static function registerTaxonomies(){

        $forumSlugBase = DSCF_DSF_SupportForums_Main::getSlugBase();

        // Register the Custom Support Forum Category Structure
        register_taxonomy(self::KEYNAME_CATEGORY, null,
            array(
                'hierarchical'      => true, // make it hierarchical (like categories)
                'labels'            => array(
                    'name'              => _x('Forum Categories', 'taxonomy general name'),
                    'singular_name'     => _x('Forum Categories', 'taxonomy singular name'),
                    'search_items'      => __('Search Forum Categories'),
                    'all_items'         => __('All Forum Categories'),
                    'parent_item'       => __('Parent Forum Category'),
                    'parent_item_colon' => __('Parent Forum Category:'),
                    'edit_item'         => __('Edit Forum Category'),
                    'update_item'       => __('Update Forum Category'),
                    'add_new_item'      => __('Add New Forum Category'),
                    'new_item_name'     => __('New Forum Category Name'),
                    'menu_name'         => __('Categories'),
                ),
                'public'            => true,
                'show_ui'           => true,
                'show_in_menu'      => true,
                'show_admin_column' => true,
                'query_var'         => true,
                'rewrite'           => ['slug' => $forumSlugBase.'/category'],
            )
        );

        // Register the Custom Support Forum Category Structure
        register_taxonomy(self::KEYNAME_TAG, null,
            array(
                'hierarchical'      => false, // make it non hierarchical (like regular tags)
                'labels'            => array(
                    'name'              => _x('Forum Tags', 'taxonomy general name'),
                    'singular_name'     => _x('Forum Tags', 'taxonomy singular name'),
                    'search_items'      => __('Search Forum Tags'),
                    'all_items'         => __('All Forum Tags'),
                    'parent_item'       => __('Parent Forum Tag'),
                    'parent_item_colon' => __('Parent Forum Tag:'),
                    'edit_item'         => __('Edit Forum Tag'),
                    'update_item'       => __('Update Forum Tag'),
                    'add_new_item'      => __('Add New Forum Tag'),
                    'new_item_name'     => __('New Forum Tag Name'),
                    'menu_name'         => __('Tags'),
                ),
                'public'            => true,
                'show_ui'           => true,
                'show_in_menu'      => true,
                'show_admin_column' => true,
                'query_var'         => true,
                'rewrite'           => ['slug' => $forumSlugBase.'/tag'],
            )
        );

        // Register the Custom Forum Post Type
        register_post_type( self::KEYNAME,
            array(
                'labels'                => array(
                    'name'                  => __( 'Forum Threads' ),
                    'singular_name'         => __( 'Forum Thread' ),
                    'menu_name'             => __( 'Support Forums' ),
                    'all_items'             => __( 'All Forum Threads' ),
                    'view_item'             => __( 'View Forum Thread' ),
                    'view_items'            => __( 'View Forum Threads' ),
                    'archives'              => __( 'Forum Threads' ),
                    'add_new_item'          => __( 'Add New Forum Thread' ),
                    'edit_item'             => __( 'Edit Forum Thread' ),
                    'update_item'           => __( 'Update Forum Thread' ),
                    'search_items'          => __( 'Search Forum Threads' ),
                ),
                'description'           => __( 'Dimbal Support Forum Threads' ),
                'public'                => true,
                'has_archive'           => true,
                'rewrite'               => array('slug' => $forumSlugBase.'/thread'),
                'hierarchical'          => false,
                'supports'              => array( 'title' ),
                'show_ui'               => true,
                'show_in_menu'          => strtolower(DSCF_DSF_SupportForums_Thread::KEYNAME),
                'show_in_nav_menus'     => true,
                'show_in_admin_bar'     => false,
                'can_export'            => true,
                'exclude_from_search'   => false,
                'publicly_queryable'    => true,
                'capability_type'       => 'post',
                'taxonomies'            => array( self::KEYNAME_CATEGORY, self::KEYNAME_TAG ),
            )
        );

        // Associate the Taxonomies with the Object Types
        register_taxonomy_for_object_type( self::KEYNAME_TAG, self::KEYNAME );
        register_taxonomy_for_object_type( self::KEYNAME_CATEGORY, self::KEYNAME );

    }

    public static function editorBuildOptions($object){

        $options=array();
        $options[]=array(
            'title'=>'Description',
            'objectType'=>DSCF_DSF_StandardEditor::OT_STRING,
            'objectName'=>'description',
            'formType'=>DSCF_DSF_StandardEditor::ET_TEXTAREA_HTML,
            'value'=>($object)?$object->description:'',
            'help'=>'The submitted description for the thread.',
        );

        return $options;
    }

    public static function get_display($content){

        $postId = get_the_ID();
        $is_single = is_single();

        $html = '';

        // Whether or not to display the Navigation Links and other Meta Fields
        if($is_single){

            $addThreadUrl=DSCF_DSF_SupportForums_Main::getAddThreadUrl();
            $forumHomeUrl = DSCF_DSF_SupportForums_Main::getHomeUrl();
            $currentUser = wp_get_current_user();

            $html .= '<div class="dsf-align-right"><a href="'.$addThreadUrl.'">Add Thread</a> | <a href="'.$forumHomeUrl.'">Forum Home</a></div>';

            // Display the categories and tags
            $html .= '<p class="entry-categories dsf-align-right">Posted in '.self::getTheTermsDisplay(array('post_id'=>$postId,'taxonomy'=>self::KEYNAME_CATEGORY, 'title'=>''));
            $html .= ' and tagged with '.self::getTheTermsDisplay(array('post_id'=>$postId,'taxonomy'=>self::KEYNAME_TAG, 'title'=>'')).'</p>';

            $html .= '<br />';

            $comments = DSCF_DSF_SupportForums_Comment::getAllByThreadId($postId);
            if(empty($comments) || !is_array($comments)){
                $comments = array();
                $html .= '<p>Be the first to reply below!</p>';
            }else{

            }

            $html .= '<div id="comments" class="comment-list">';

            $html .= '<hr />';

            foreach($comments as $commentObject){
                if(!empty($commentObject)){
                    if($commentObject->approved){

                        $createdTime = DSCF_DSF_StandardObjectRecord::formatDate($commentObject->createdDate);
                        $userString = "annonymous";
                        $user = get_userdata($commentObject->userId);
                        if(!empty($user)){
                            $userString = $user->user_login;
                        }

                        $html .= '<p class="entry-meta">Submitted by '.$userString.' on '.$createdTime.'</p>';
                        $html .= '<div class="entry-content" style="margin-left:25px; padding-left:25px; border-left:10px solid gainsboro;">'. DSCF_DSF_SupportForums_Comment::getCommentHtml($commentObject) .'</div>';

                        $html .= '</br /><hr />';

                    }

                }
            }

            $html .= '</div>';

            $html .= '<br />';

            $html .= '<h2 class="entry-title">Leave a Reply</h3>';

            // Display the add comment form
            if(empty($currentUser)){
                $html .= '<p>Please <a href="'.wp_login_url( get_permalink() ).'" title="Login">Login</a> to comment.</p>';
            }else{

                if(DSCF_DSF_SupportForums_Main::canUserAddContent($currentUser->ID)){

                    $html .= '
                    <div>
                    <form id="new_comment" name="new_comment" method="post" action="'.DSCF_DSF_SupportForums_Main::getAddCommentHandlerUrl().'">
                    <p>Logged in as: '.$currentUser->user_login.'</p>
                    <input type="hidden" name="thread_id" value="'.$postId.'" />
                    <p><label for="comment">Comment:</label><br />
                    <textarea name="comment" class="dsf-textarea"></textarea></p>
                    <div><input type="submit" name="submit" value="Submit Comment"></div>
                    <input type="hidden" name="action" value="new_comment" />
                    </form>
                    </div>
                    ';

                }else{

                    $html .= '<p>Please slow down.  Only so many posts and comments can be made in such a short timeframe.</p>';

                }
            }

            $html .= '<br />';


        }else{

            // Not a single Item - more like a list display

            $commentCount = 0;
            $comments = DSCF_DSF_SupportForums_Comment::getAllByThreadId($postId);
            $commentCount = count($comments);

            $firstComment = array_shift($comments);
            $latestComment = array_pop($comments);
            $lastPostDate = DSCF_DSF_Utilities::getFromObject('createdDate',$latestComment, null);
            $lastPostString = DSCF_DSF_StandardObjectRecord::formatDate($lastPostDate);

            $html .= 'Last Post: '.$lastPostString.' | Replies: '.$commentCount;

        }

        return $html;
    }

    public static function wpFilterManagePostsColumn($defaults) {
        $defaults['comment_count'] = 'Comments';
        return $defaults;
    }

    public static function wpFilterManagePostsCustomColumn($columnName, $postId){

        //check for the right post type
        if ($columnName == 'comment_count'){

            // TODO -- Need to cache these results as this will be poor performance

            $comments = DSCF_DSF_SupportForums_Comment::getAllByThreadId($postId);
            $approved = 0;
            $pending = 0;
            foreach($comments as $comment){
                if($comment instanceof DSCF_DSF_SupportForums_Comment){
                    if($comment->isApproved()){
                        $approved++;
                    }else{
                        $pending++;
                    }
                }
            }

            echo $pending.'/'.$approved;
        }

    }

    public static function getTableDisplayForThreads($posts){

        $html = '';

        foreach($posts as $postObject){

            //setup_postdata($postObject);

            $comments = DSCF_DSF_SupportForums_Comment::getAllByThreadId($postObject->ID);
            $commentCount = count($comments);

            $latestComment = array_pop($comments);
            $lastPostDate = DSCF_DSF_Utilities::getFromObject('createdDate',$latestComment, null);
            $lastPostString = DSCF_DSF_StandardObjectRecord::formatDate($lastPostDate);

            $categoryParams = array('post_id'=>$postObject->ID,'taxonomy'=>self::KEYNAME_CATEGORY, 'title'=>'', 'separator'=>', ');
            $tagParams = array('post_id'=>$postObject->ID,'taxonomy'=>self::KEYNAME_TAG, 'title'=>'', 'separator'=>', ');

            $html .= '
                <div>
                    <div><a href="'.get_permalink($postObject->ID).'">'.$postObject->post_title.'</a> | Last Post: '.$lastPostString.' | Replies: '.$commentCount.'</div>
                    <!--<div>Last Post: '.$lastPostString.' | Replies: '.$commentCount.' | Posted in '.self::getTheTermsDisplay($categoryParams).' and tagged with '.self::getTheTermsDisplay($tagParams).'</div>-->
                </div>
            ';

        }

        return $html;

    }

    public static function getVirtualPagesForumHome($params=array()){

        // Check to see if the Plugin is Active
        if(!DSCF_DSF_SupportForums_Main::isPluginEnabled()){
            wp_redirect( home_url() );
            exit;
        }

        // Enqueue Scripts
        DSCF_DSF_SupportForums_Main::wpActionEnqueueScripts();

        $html = '';

        $addThreadUrl=DSCF_DSF_SupportForums_Main::getAddThreadUrl();
        $html .= '
        <p class="dsf-align-right">
            <a href="'.$addThreadUrl.'">Add Thread</a>
        </p>
        ';

        // Categories
        $terms = get_terms( array(
            'taxonomy' => DSCF_DSF_SupportForums_Thread::KEYNAME_CATEGORY,
            'hide_empty' => false,
        ) );

        if(!empty($terms)){
            $html .= '<h2>Topic Category List</h2>';
            $html .= '<div class="dsf-div-table">';

            foreach($terms as $term){
                $html .= '<div class="dsf-div-table-row">';
                $html .= '<div class="dsf-div-table-cell"><a href="'.get_term_link($term).'">'.$term->name.'</a></div>';
                //$html .= '<div class="dsf-div-table-cell dsf-align-right">'.$post->comment_count.'</div>';
                $html .= '</div>';
            }

            $html .= '</div>';
            $html .= '<br />';
        }

        // Recent Posts
        if(!is_array($params)){
            $params = array();
        }

        $query_args = array(
            'numberposts'=>10,
            'post_status' => 'publish',
            'post_type'  => DSCF_DSF_SupportForums_Thread::KEYNAME,
        );

        $posts = get_posts($query_args);

        $html .= '<h2>Recent Forum Threads</h2>';
        $html .= self::getTableDisplayForThreads($posts);


        $details = array(
            'title'=>'Support Forums Home',
            'content'=>$html
        );

        return $details;

    }

    public static function getVirtualPagesForumThreadListForTerm($params=array()){

        // Check to see if the Plugin is Active
        if(!DSCF_DSF_SupportForums_Main::isPluginEnabled()){
            wp_redirect( home_url() );
            exit;
        }

        // Enqueue Scripts
        DSCF_DSF_SupportForums_Main::wpActionEnqueueScripts();

        if(!is_array($params)){
            $params = array();
        }

        $addThreadUrl=DSCF_DSF_SupportForums_Main::getAddThreadUrl();
        $forumHomeUrl = DSCF_DSF_SupportForums_Main::getHomeUrl();
        $title="";

        $query_args = array(
            'numberposts'=>500,
            'post_status' => 'publish',
            'post_type'  => DSCF_DSF_SupportForums_Thread::KEYNAME,
        );

        $termType = null;

        if(array_key_exists('category_slug', $params)){
            $slug = $params['category_slug'];
            $termType = DSCF_DSF_SupportForums_Thread::KEYNAME_CATEGORY;
        }elseif(array_key_exists('tag_slug', $params)){
            $slug = $params['tag_slug'];
            $termType = DSCF_DSF_SupportForums_Thread::KEYNAME_TAG;
        }

        $html = '';

        if(!empty($slug)){

            $term = get_term_by('slug', $slug, $termType);
            if(!empty($term)){
                $query_args['tax_query']=array(array(
                    'taxonomy' => $termType,
                    'field' => 'term_id',
                    'terms' => $term->term_id,
                ));
                $title = " : ".$term->name;

            }
        }

        $posts = get_posts($query_args);

        $html .= '
                <p class="dsf-align-right">
                    <a href="'.$addThreadUrl.'">Add Thread</a> | <a href="'.$forumHomeUrl.'">Forum Home</a>
                </p>
                ';

        $html .= self::getTableDisplayForThreads($posts);

        $details = array(
            'title'=>'Forum Threads'.$title,
            'content'=>$html
        );

        return $details;

    }

}
