<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/31/15
 * Time: 10:01 AM
 * To change this template use File | Settings | File Templates.
 */
class DSCF_DSF_Tag extends DSCF_DSF_StandardObjectRecord{

    public $text;
    public $items=array();

    const TABLE_NAME = "tag";

    public function __construct(){

        global $wpdb;
        $table_name = self::getTableName();

        //Pass it to the parent to setup some common items
        $this->create();

        //Save in the DB
        try{
            $wpdb->insert(
                $table_name,
                array(
                    'blogId' => $this->blogId,
                    'data' => self::pack($this)
                ),
                array(
                    '%d',
                    '%s'
                )
            );
        }catch(Exception $e){
            //DSCF_DSF_Utilities::logMessage('Error creating Poll: '.$e->getMessage());
            return false;
        }

        //Get the ID of the inserted row and save it back to the object
        try{
            $this->id = $wpdb->insert_id;
            $this->save();
        }catch(Exception $e){
            //DSCF_DSF_Utilities::logMessage('Error creating Poll: '.$e->getMessage());
            return false;
        }

        //Return the object
        return $this;
    }

    public function save(){
        global $wpdb;
        $table_name = self::getTableName();
        $this->setLastModified();
        try{
            $wpdb->update(
                $table_name,
                array(
                    'data' => self::pack($this)
                ),
                array( 'ID' => $this->id ),
                array(
                    '%s'
                ),
                array( '%d' )
            );
        }catch(Exception $e){
            //DSCF_DSF_Utilities::logMessage('Error saving Object: '.$e->getMessage());
            return false;
        }
        return true;
    }

    // Controlled by Parent Class
    //public static function get($id){}

    // Controlled by Parent Class
    //public static function getAll($start=0,$limit=500){}

    // Controlled by Parent Class
    //public static function deleteById($id){}

    /*
     * Add a bunch of new items for a given Tag
     */
    public function addItemIds($itemIds){
        foreach($itemIds as $itemId){
            $this->items[] = $itemId;
            $link = new DSCF_DSF_TagItem($itemId, $this->id);
        }
        $this->save();
    }

    public function getItemIds(){
        $records = DSCF_DSF_TagItem::getAllForTagId($this->id);
        $itemIds = array();
        foreach($records as $record){
            $itemIds[] = $record['itemId'];
        }

        error_log("RECORDS: ".print_r($records, true));
        error_log("ITEMIDS: ".print_r($itemIds, true));

        return $itemIds;
    }

    public function removeItemIds($itemIds){
        foreach($itemIds as $itemId){
            DSCF_DSF_TagItem::deleteSingleRelationship($itemId, $this->id);
        }
    }

    public function removeAllItemIds(){
        DSCF_DSF_TagItem::deleteAllForTagId($this->id);
    }

    public static function getTagsForItem($itemId){
        $tags = array();
        $records = DSCF_DSF_TagItem::getAllForItemId($itemId);
        foreach($records as $record){
            if(array_key_exists('tagId', $record)){
                $tags[] = $record['tagId'];
            }
        }
        return $tags;
    }

    public static function getTagObjectsForItem($itemId){
        $tagIds = DSCF_DSF_TagItem::getAllForItemId($itemId);
        $objects = array();
        foreach($tagIds as $tagId){
            $tag = self::get($tagId);
            if(!empty($tag)){
                $objects[]=$tag;
            }
        }
        return $objects;
    }

    public static function addTagsForItem($itemId, $tagIds){
        foreach($tagIds as $tagId){
            $link = new DSCF_DSF_TagItem($itemId, $tagId);
        }
    }

    public static function removeTagsForItem($itemId, $tagIds){
        foreach($tagIds as $tagId){
            DSCF_DSF_TagItem::deleteSingleRelationship($itemId, $tagId);
        }
    }

    public static function removeAllTagsForItem($itemId){
        DSCF_DSF_TagItem::deleteAllForItemId($itemId);
    }


    public static function editorBuildOptions($object){
        $tagTypeId = self::getTypeIdByAppCode(WPDEVHUB_CONST_DSF_APP_CODE);
        $options=array();
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Basic '.WPDEVHUB_CONST_DSF_ZONE_GROUP_NAME.' Properties',
        );
        $options[]=array(
            'title'=>'ID',
            'objectType'=>DSCF_DSF_StandardEditor::OT_NUMERIC,
            'objectName'=>'id',
            'formType'=>DSCF_DSF_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?$object->id:'',
            'help'=>''
        );
        $options[]=array(
            'title'=>'Title',
            'objectType'=>DSCF_DSF_StandardEditor::OT_STRING,
            'objectName'=>'text',
            'formType'=>DSCF_DSF_StandardEditor::ET_TEXT,
            'value'=>($object)?$object->text:'',
            'help'=>'The Name of the Tag',
            'size'=>100,
        );

        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Additional Information',
        );
        $options[]=array(
            'title'=>'Created Date',
            'objectType'=>DSCF_DSF_StandardEditor::OT_SKIP,
            'objectName'=>'createdDate',
            'formType'=>DSCF_DSF_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?DSCF_DSF_StandardObjectRecord::formatDate($object->createdDate):'',
            'help'=>'The date the '.WPDEVHUB_CONST_DSF_ZONE_GROUP_NAME.' was created on'
        );
        return $options;
    }

    public static function managerBuildOptions($objects){
        $rows = array();
        foreach($objects as $object){
            $row = array();
            $row[] = array(
                'title'=>'ID',
                'content'=>$object->id,
            );
            $row[] = array(
                'title'=>'Status',
                'content'=>$object->getFormattedStatusImage($object->status, '24px'),
            );
            $row[] = array(
                'title'=>'Edit',
                'url'=>DSCF_DSF_Utilities::getPageUrl(WPDEVHUB_CONST_DSF_PAGE_ZONES, array('id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DSF_URL_IMAGES.'/page_edit.png',
                'image_tooltip'=>'Edit '.WPDEVHUB_CONST_DSF_ZONE_GROUP_NAME,
            );
            $row[] = array(
                'title'=>'Text',
                'content'=>$object->text,
            );

            $row[] = array(
                'title'=>'Created Date',
                'content'=>DSCF_DSF_StandardObjectRecord::formatSiteDate($object->createdDate),
            );
            $row[] = array(
                'title'=>'Delete',
                'url'=>DSCF_DSF_Utilities::getPageUrl(WPDEVHUB_CONST_DSF_PAGE_ZONES, array('delete'=>1,'id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DSF_URL_IMAGES.'/delete.png',
                'image_tooltip'=>'Delete '.WPDEVHUB_CONST_DSF_ZONE_GROUP_NAME,
            );
            $rows[]=$row;
        }
        return $rows;
    }

    public function getPreviewUrl(){
        return DSCF_DSF_Utilities::getPageUrl(WPDEVHUB_CONST_DSF_PAGE_PREVIEW, array('ac'=>'3','tagId'=>$this->id));
    }

    public function getShortcode(){
        return self::buildShortcodeHelper(array('tag_id'=>$this->id));
    }

    public static function getTagManagerPageContent(){

        $html = "";

        // Build the Header
        $html .= DSCF_DSF_Utilities::buildHeader(array(
            'title'=>'Tag Manager',
            'icon'=>WPDEVHUB_CONST_DSF_URL_IMAGES.'/bricks.png',
            'description'=>'Use this manager to build and maintain custom Tags for use in this plugin.',
            'buttons'=>array(
                0=>array('text'=>'Create New Tag','params'=>array('page'=>DSCF_DSF_Utilities::buildPageSlug(WPDEVHUB_CONST_DSF_PAGE_TAG), 'id'=>'new')),
                1=>array('text'=>'View All','params'=>array('page'=>DSCF_DSF_Utilities::buildPageSlug(WPDEVHUB_CONST_DSF_PAGE_TAG))),
            )
        ));

        // Check for a delete request
        $html .= DSCF_DSF_Tag::checkForDelete(array());

        ///////////////////////  Editor DISPLAY  ///////////////////////////
        $html .= DSCF_DSF_StandardEditor::buildPageTemplate(DSCF_DSF_Utilities::buildAppClassName('DSCF_DSF_Tag'));

        // If the ID field was removed or is not present that means we want the Manager
        $id = DSCF_DSF_Utilities::getRequestVarIfExists('id');
        if(empty($id)){
            ///////////////////////  MANAGER DISPLAY  ///////////////////////////
            $rows = DSCF_DSF_Tag::managerBuildOptions(DSCF_DSF_Tag::getAll());
            $html .= DSCF_DSF_StandardManager::buildManagerTable($rows);
        }

        // Close the wrapper
        $html .= DSCF_DSF_Utilities::buildFooter();

        return $html;

    }

}
