


// Jquery routine to initialize the Dimbal Object
jQuery(document).ready(function($) {
    DSCF_DSF_Quiz.initialize();
});

function DSCF_DSF_QuizAnswerObject(questionId, answerChoice){
    this.questionId = questionId;
    this.answerChoice = answerChoice;
}

function DSCF_DSF_QuizObject(){
    this.quizId = 0;
    this.quizObject = null;
    this.questions = [];
    this.answers=[];
    this.elementId=null;
    this.resultHash="";
    this.percentComplete=0;
    this.percentCorrect=0;

    this.getNextQuizStep=function(){

        console.log("Inside getNextQuizStep");

        // Get a question that has not yet been answered
        var question = null;
        for(var q in this.questions){
            var answerFound = false;
            var questionId = this.questions[q].id;
            for(var a in this.answers){
                if(questionId == this.answers[a].questionId){
                    // Question already has an answer
                    answerFound = true;
                }
            }
            if(answerFound){
                // Do not include this question
            }else if(this.questions[q].choices == undefined || this.questions[q].choices.length <= 0){
                // Skip this question - it has no valid answer choices
                console.log("Skipping question because it has no answer chocies");
                console.dir(this.questions[q]);
            }else{
                question = this.questions[q];
                break;
            }
        }

        if(question == null){
            // We are done with the quiz
            this.displayQuizEnding();
        }else{
            this.displayQuestion(question);
        }

    };

    this.displayQuestion=function(question){
        console.log("Inside displayQuestion");
        console.dir(this);
        var html='';

        html += '<div class="quizQuestionWrapper">';
        html += '<div class="quizQuestionText">'+question.question+'</div><br />';
        for(var c in question.choices){
            var choice = question.choices[c];
            html += '' +
                '<div class="quizQuestionOptionText"><input type="radio" name="answerChoice" value="'+c+'" /> '+choice+'</div>';
        }

        html += '<br /><br />';
        html += '<button onclick="DSCF_DSF_Quiz.submitResponse('+this.id+','+question.id+')">Save Answer</button>';
        html += '</div>';

        if(this.resultHash != null && this.url != undefined){
            // Display the result hash prompt
            var returnUrl = this.url + this.resultHash;
            html += '<br />';
            html += '<div class="dsf-align-center">Continue this quiz later: <input type="text" readonly size="50" value="'+returnUrl+'" /></div>'
        }

        jQuery('#'+this.elementId).html(html);
    };

    this.displayQuizEnding=function(){
        console.log("Inside displayQuizEnding");
        console.dir(this);
        // At this point I want to display the details about the results

        var html = '';

        html += '<div class="quizResultsWrapper">';
        html += '<p>Quiz Completed.</p>';
        html += '<div class="dsf-bold">Score: '+this.percentCorrect+'%</div>';
        html += '<hr />';

        for(q in this.questions){
            var question = this.questions[q];

            html += '<div class="quizResultsRowWrapper">';

            html += '<div>'+question.question+'</div>';

            for(a in this.answers){
                if(this.answers[a].questionId == question.id){
                    var selectedAnswer = this.answers[a].answerChoice;

                    for(c in question.choices){
                        if(selectedAnswer == c){
                            var answerText = question.choices[c];
                        }
                    }
                    html += '<br />';

                    if(selectedAnswer == question.selectedAnswer){
                        // User got it correct
                        html += '<p class="dsf-highlight-box"><img src="'+DSCF_DSF_Quiz.url_root+'/images/accept.png" class="dsf-icon24" alt="Correct" /> Correct</p>';
                        html += '<p class="dsf-italic">Your Answer: '+answerText+'</p>';
                        if(question.explanationCorrect != ""){
                            html += '<div>'+question.explanationCorrect+'</div>';
                        }
                    }else{
                        // User got it wrong
                        html += '<p class="dsf-highlight-box"><img src="'+DSCF_DSF_Quiz.url_root+'/images/exclamation.png" class="dsf-icon24" alt="Incorrect" /> Incorrect</p>';
                        html += '<p class="dsf-italic">Your Answer: '+answerText+'</p>';
                        if(question.explanationIncorrect != ""){
                            html += '<div>'+question.explanationIncorrect+'</div>';
                        }
                    }


                }


            }

            html += '</div>';
            html += '<hr />';

        }


        html += '</div>';

        jQuery('#'+this.elementId).html(html);

    }
}

// Dimbal Quiz Object - should rename this object to dpm
var DSCF_DSF_Quiz = {

    initialized:false,
    slug:"WPDEVHUB_CONST_DSF_SLUG",
    url_root:"",
    url_ajax:"",
    url_ajax_load:"",
    blankCounter:0,

    quizzes:[],

    initialize:function() {

        if(this.initialized){
            // Already initialized
            return;
        }


        //Setup a get Elements by ClassName function if one does not exist
        if (document.getElementsByClassName == undefined) {
            document.getElementsByClassName = function(className)
            {
                var hasClassName = new RegExp("(?:^|\\s)" + className + "(?:$|\\s)");
                var allElements = document.getElementsByTagName("*");
                var results = [];

                var element;
                for (var i = 0; (element = allElements[i]) != null; i++) {
                    var elementClass = element.className;
                    if (elementClass && elementClass.indexOf(className) != -1 && hasClassName.test(elementClass))
                        results.push(element);
                }
                return results;
            }
        }

        // Setup the URLS
        this.url_root = dimbal_dqq_vars.url;
        this.url_ajax = dimbal_dqq_vars.url_ajax;
        this.url_ajax_load = this.url_root+"/images/loading2.gif";

        //Get the initial elements
        var elements = document.getElementsByClassName(this.slug+"-WidgetWrapper");
        for(var i=0;i<elements.length;i++){
            //Get needed attributes
            var dqqQueryString;
            var element = elements[i];
            var dqqQuiz = element.getAttribute("dqq_quiz");
            var elementId = element.getAttribute("id");

            //First enter an ID if one does not exist (required for Ajax response)
            if(elementId == undefined || elementId == ""){
                elementId = this.slug+"_"+i;
                elements[i].setAttribute("id", elementId);
            }

            // Setup the proper Display Ajax Action
            var queryParams = {
                action: DSCF_DSF_Quiz.slug+"-display-quiz",
                dqqEId: elementId
            };



            //Now get the Poll or Zone
            if(dqqQuiz != undefined && dqqQuiz != "" && dqqQuiz != null){
                queryParams.quizId = dqqQuiz;
            }

            console.log("Quiz Object About to Start AJAX");
            console.dir(DSCF_DSF_Quiz);
            console.dir(queryParams);

            //Now make the Ajax Request
            jQuery.post(DSCF_DSF_Quiz.url_ajax, queryParams, function( response ){
                DSCF_DSF_Quiz.generalAjaxResponse( response );
            });

        }

        // Mark this object as initialized
        this.initialized = true;
    },

    getQuizObject:function(quizId){
        for(q in DSCF_DSF_Quiz.quizzes){
            if(DSCF_DSF_Quiz.quizzes[q].id == quizId){
                return DSCF_DSF_Quiz.quizzes[q];
            }
        }
    },

    submitResponse:function( quizId, questionId ){

        //console.log("Inside submitQuestion");

        var quizObject = this.getQuizObject(quizId);

        //console.log("Quiz Object Found");
        //console.dir(quizObject);

        var answerChoice = jQuery("input:radio[name='answerChoice']:checked").val();

        var answerObject = new DSCF_DSF_QuizAnswerObject(questionId, answerChoice);

        quizObject.answers.push(answerObject);

        // Setup the proper Display Ajax Action
        var queryParams = {
            action: DSCF_DSF_Quiz.slug+"-save-response",
            resultHash: quizObject.resultHash,
            quizId: quizId,
            questionId: questionId,
            answerChoiceId: answerChoice
        };

        console.log("Making the AJAX call in submitResponse");
        console.dir(queryParams)

        //Now make the Ajax Request
        jQuery.post(DSCF_DSF_Quiz.url_ajax, queryParams, function( response ){
            // Do I need to do anything with this -- not sure that I do...

            console.log("Inside SaveResponse return");
            console.log(response);

            if(response != undefined && response.quizId != undefined){
                for(q in DSCF_DSF_Quiz.quizzes){
                    if(DSCF_DSF_Quiz.quizzes[q].id == response.quizId){
                        // This is the active quiz to work with
                        var quizObject = DSCF_DSF_Quiz.quizzes[q];

                        if(response.percentComplete != undefined){
                            quizObject.percentComplete = response.percentComplete;
                        }

                        if(response.percentCorrect != undefined){
                            quizObject.percentCorrect = response.percentCorrect;
                        }

                        if(quizObject.resultHash == "" && response.resultHash != undefined){
                            quizObject.resultHash = response.resultHash;
                        }

                        console.log("Save was successful...");
                        console.dir(response);

                        // Go to the next step in the quiz
                        quizObject.getNextQuizStep();

                    }
                }
            }else{
                console.log("Response had undefined data");
            }

        });

    },

    generalAjaxResponse:function( response ){

        if(response == undefined || response.dqqEId == undefined || response.quiz == undefined || response.questions == undefined){
            // Bad response
            console.log("Bad Data inside generalAjaxResponse");
            console.dir(response);
        }else{

            console.log("Good Data inside generalAjaxResponse");
            console.dir(response);

            var dqqEId = response.dqqEId;
            var quiz = new DSCF_DSF_QuizObject();
            quiz.questions = response.questions;
            quiz.quizObject = response.quiz;
            quiz.id = response.quiz.ID;
            quiz.elementId = dqqEId;

            quiz.getNextQuizStep();

            DSCF_DSF_Quiz.quizzes.push(quiz);

        }

    },

    addBlankRow:function(objectName, beforeId){
        this.blankCounter++;
        var textObjectName = objectName+'_blankTxt_'+this.blankCounter;
        var html = '<div id="'+textObjectName+'_wrapper">';
        html += '<input type="radio" name="'+objectName+'_rdo" value="'+this.blankCounter+'" />';
        html += '<input type="text" name="'+textObjectName+'" value="" size="50" /><button type=button onclick="DSCF_DSF_Quiz.deleteOptionRow(\''+textObjectName+'\')">Delete</button>';
        html += '</div>';
        jQuery("#"+beforeId).before(html);
    },

    deleteOptionRow:function(objectName){
        jQuery("#"+objectName).val("");
        jQuery("#"+objectName+"_wrapper").remove();
    },
    setCounter:function(counter){
        if(this.blankCounter == 0){
            this.blankCounter = counter;
        }
    }

}
