<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/18/15
 * Time: 8:47 PM
 * To change this template use File | Settings | File Templates.
 */

// Check to see if the Plugin is Active
if(!DSCF_DSF_SupportForums_Main::isPluginEnabled()){
    wp_redirect( home_url() );
    exit;
}

// Validate this was a POST
if($_SERVER['REQUEST_METHOD'] != 'POST'){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Request was not a POST request type.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(121) );
    exit;
}

DSCF_DSF_Utilities::logMessage("POST PARAMS: ".print_r($_POST, true));

// Validate the action param was passed
if(empty( $_POST['action'] ) || $_POST['action'] != "new_comment"){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Post action param is invalid.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(121) );
    exit;
}

// Get the user object
$userObject = wp_get_current_user();
if(empty($userObject) || empty($userObject->ID)){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Could not get logged in User.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(124) );
    exit;
}

// Validate the user can add content (throttling)
if(!DSCF_DSF_SupportForums_Main::canUserAddContent($userObject->ID)){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. User Validation did not pass.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(125) );
    exit;
}

// Validate the thread is available to be created (user account is of high enough status, etc)
$threadValidation = DSCF_DSF_SupportForums_Main::checkThreadValidation();
if(!$threadValidation){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Did not pass Thread Validation.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(126) );
    exit;
}

// Validate the comment
$comment = DSCF_DSF_Utilities::getRequestVarIfExists("comment");
if(empty($comment)){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Comment was passed as Empty.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(122) );
    exit;
}

// Validate the thread id
$thread_id = DSCF_DSF_Utilities::getRequestVarIfExists("thread_id");
if(empty($thread_id)){
    DSCF_DSF_Utilities::logMessage("Cannot create Comment. Thread ID was passed as Empty.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(123) );
    exit;
}

// Get the default post status
$approvalStatus = DSCF_DSF_StandardSetting::getSetting(DSCF_DSF_SupportForums_Main::SETTINGS_DEFAULT_COMMENT_STATUS);
if(empty($approvalStatus)){
    $approvalStatus = DSCF_DSF_SupportForums_Comment::STATE_UNAPPROVED;
}

// Create the custom post data
$commentObject = new DSCF_DSF_SupportForums_Comment();
$commentObject->comments = $comment;
$commentObject->threadId = $thread_id;
$commentObject->userId = $userObject->ID;
$commentObject->approved = $approvalStatus;
$commentObject->save();

// Save the history of the user submitting a thread
DSCF_DSF_SupportForums_Main::addUserContentHistory($userObject->ID);


// Send the moderation email
$editLink = get_edit_post_link($thread_id);
$options = array(
    'subject'=>'Please moderate Thread Comment',
    'message'=>'The following forum thread has received a new comment:<br /><a href="'.$editLink.'">'.$editLink.'</a><br />============================<br /><br />'.$comment,
);
DSCF_DSF_SupportForums_Main::sendModeratorNotification($options);


$redirectUrl = DSCF_DSF_SupportForums_Main::getAddContentConfirmationUrl();

if($approvalStatus){
    // Topic is being immediately publish - send them to the thread
    $postUrl = get_permalink($thread_id);
    if(empty($postUrl)){
        DSCF_DSF_Utilities::logMessage("Error in redirection after custom thread creation.  URL was empty from new Thread.  Sending to generic confirmation page");
    }else{
        $redirectUrl = $postUrl;
    }
}

// Now perform the redirection
//wp_redirect( $redirectUrl . "?postid=".$thread_id );
wp_redirect( $redirectUrl );
exit;
