<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/18/15
 * Time: 8:47 PM
 * To change this template use File | Settings | File Templates.
 */

// Check to see if the Plugin is Active
if(!DSCF_DSF_SupportForums_Main::isPluginEnabled()){
    wp_redirect( home_url() );
    exit;
}

// Validate this was a POST
if($_SERVER['REQUEST_METHOD'] != 'POST'){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Request was not a POST request type.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(101) );
    exit;
}

// Validate the action param was passed
if(empty( $_POST['action'] ) || $_POST['action'] != "new_post"){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Post action param is invalid.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(101) );
    exit;
}

// Get the user object
$userObject = wp_get_current_user();
if(empty($userObject) || empty($userObject->ID)){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Could not get logged in User.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(104) );
    exit;
}

// Validate the user can add content (throttling)
if(!DSCF_DSF_SupportForums_Main::canUserAddContent($userObject->ID)){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. User Validation did not pass.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(105) );
    exit;
}

// Validate the thread is available to be created (user account is of high enough status, etc)
$threadValidation = DSCF_DSF_SupportForums_Main::checkThreadValidation();
if(!$threadValidation){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Did not pass Thread Validation.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(106) );
    exit;
}

// Validate the title
$title = DSCF_DSF_Utilities::getRequestVarIfExists("title");
if(empty($title)){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Title was passed as Empty.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(102) );
    exit;
}

// Validate the content... Only do so if the settings made it so
$content = DSCF_DSF_Utilities::getRequestVarIfExists("content");
$allowEmptyContent = DSCF_DSF_StandardSetting::getSetting(DSCF_DSF_SupportForums_Main::SETTINGS_ALLOW_EMPTY_CONTENT);
if(empty($content) && empty($allowEmptyContent)){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Content was passed as Empty.");
    wp_redirect( DSCF_DSF_SupportForums_Main::getErrorHandlerUrl(103) );
    exit;
}


// Validate the category
$categoryId = DSCF_DSF_Utilities::getRequestVarIfExists("cat");

// Validate the tags
$tagString = DSCF_DSF_Utilities::getRequestVarIfExists("tags");


// Now fix up the content a little (might not need to do this since I am now using custom posts.. let's see
//$title = substr($title, 0, DSCF_DSF_SupportForums_Thread::getThreadTitleMaxLength());
//$title = stripslashes($title);
//$content = stripslashes($content);
//$title = DSCF_DSF_SupportForums_Main::cleanAndSanitizeUserInput($title);
//$content = DSCF_DSF_SupportForums_Main::cleanAndSanitizeUserInput($content);

// Get the default post status
//$postStatus = DSCF_DSF_StandardSetting::getSetting(DSCF_DSF_SupportForums_Main::SETTINGS_DEFAULT_THREAD_STATUS);
//if(empty($postStatus)){
    $postStatus = "draft";
//}

// Get the default comment status
$approvalStatus = DSCF_DSF_StandardSetting::getSetting(DSCF_DSF_SupportForums_Main::SETTINGS_DEFAULT_COMMENT_STATUS);
if(empty($approvalStatus)){
    $approvalStatus = DSCF_DSF_SupportForums_Comment::STATE_UNAPPROVED;
}
if($approvalStatus == DSCF_DSF_SupportForums_Comment::STATE_APPROVED){
    $postStatus = "publish";
}

// Setup the post vars
$new_post = array(
    'post_author'   => $userObject->ID,
    'post_title'    => $title,
    'post_content'  => '',
    'post_status'   => $postStatus,           // Choose: publish, preview, future, draft, etc.
    'post_type'     => DSCF_DSF_SupportForums_Thread::KEYNAME,
    'comment_status'=> DSCF_DSF_StandardSetting::getSetting(DSCF_DSF_SupportForums_Main::SETTINGS_DEFAULT_COMMENT_STATUS),
);

DSCF_DSF_Utilities::logMessage("New Post Details: ".print_r($new_post, true));

// Create the Post
$pid = wp_insert_post($new_post);

// Add the post tags and categories
wp_set_post_terms( $pid, $categoryId, DSCF_DSF_SupportForums_Thread::KEYNAME.'_category', true );
wp_set_post_terms( $pid, $tagString, DSCF_DSF_SupportForums_Thread::KEYNAME.'_tag', true );


// Save the history of the user submitting a thread
DSCF_DSF_SupportForums_Main::addUserContentHistory($userObject->ID);

// Now get the created post
$newPost = get_post($pid);

if(empty($newPost)){
    DSCF_DSF_Utilities::logMessage("Cannot create Forum Thread. Error creating Thread object.");
    wp_redirect( home_url() );
    exit;
}

// Save the comment
// Create the custom post data
if(!empty($content)){
    $commentObject = new DSCF_DSF_SupportForums_Comment();
    $commentObject->comments = $content;
    $commentObject->threadId = $pid;
    $commentObject->userId = $userObject->ID;
    $commentObject->approved = $approvalStatus;
    $commentObject->save();
}


// Send the moderation email
$editLink = get_edit_post_link($pid);
$options = array(
    'subject'=>'Please moderate: '.$title,
    'message'=>'The following forum thread has been submitted:<br /><a href="'.$editLink.'">'.$editLink.'</a><br />============================<br /><br />'.$content,
);
DSCF_DSF_SupportForums_Main::sendModeratorNotification($options);

// Prep the redirect URL
$redirectUrl = DSCF_DSF_SupportForums_Main::getAddContentConfirmationUrl();

if($postStatus == "publish"){
    // Topic is being immediately publish - send them to the thread
    $postUrl = get_permalink($pid);
    if(empty($postUrl)){
        DSCF_DSF_Utilities::logMessage("Error in redirection after custom thread creation.  URL was empty from new Thread.  Sending to generic confirmation page");
    }else{
        $redirectUrl = $postUrl;
    }
}

// Now perform the redirection
wp_redirect( $redirectUrl );
exit;
